#include <U8g2lib.h>
#include <Wire.h>
#include <driver/i2s.h>
#define I2S_WS 8
#define I2S_SCK 9
#define I2S_SD 7
#define I2S_PORT I2S_NUM_0 //choosing I2S0 controller 
#define bufferLen 64

int16_t sBuffer[bufferLen]; //int16_t is data type for 2 byte signed integers
long dt;
long vert;
long mouthC; //Center of ellipse of mouth
long mouthV; //Radius of ellipse of mouth in Y axis

U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, U8X8_PIN_NONE);

void i2s_install(){

  const i2s_config_t i2s_config = {
    .mode = i2s_mode_t(I2S_MODE_MASTER | I2S_MODE_RX),
    .sample_rate = 44100,
    .bits_per_sample = i2s_bits_per_sample_t(16),
    //Configures I2S to use only the left channel, since we have grounded the L/R pin. To use ..ONLY_RIGHT, leave L/R pin floating
    .channel_format = I2S_CHANNEL_FMT_ONLY_LEFT, 
    .communication_format = i2s_comm_format_t(I2S_COMM_FORMAT_STAND_I2S),
    .intr_alloc_flags = 0, // default interrupt priority
    .dma_buf_count = 8,
    .dma_buf_len = bufferLen,
    .use_apll = false
  };

i2s_driver_install(I2S_PORT, &i2s_config, 0, NULL);
}

void i2s_setpin(){
  const i2s_pin_config_t pin_config = {
    .bck_io_num = I2S_SCK,
    .ws_io_num = I2S_WS,
    .data_out_num = -1,
    .data_in_num = I2S_SD
  };

i2s_set_pin(I2S_PORT, &pin_config);
}


void setup(){
  Serial.begin(115200);
  pinMode(LED_BUILTIN,OUTPUT);

  if (!i2s_install() && i2s_setpin() && i2s_start(I2S_PORT)){
    Serial.println("I2S Setup not successful");
    for(;;);
  }
  
  else {
    Serial.println("I2S Setup successful ...");
  }

  delay(1000);

  if (!u8g2.begin()){
  Serial.println("Display not initialized");
  for(;;);
  }
  else {
     Serial.println("Display initialized");
  }
  delay(1000);
}

void mouth_open(){
  u8g2.clearBuffer();
  u8g2.setFontMode(1);
  u8g2.setBitmapMode(1);
  u8g2.drawFilledEllipse(83, vert, 8, 13);
  u8g2.drawFilledEllipse(62, mouthC, 8, mouthV);
  u8g2.drawFilledEllipse(41, vert, 8, 13);
  u8g2.sendBuffer();
}

void loop(){
  vert = random(30,33);
  dt = random(0,150);
  mouthC = random (49,54);
  mouthV = random (2,6);

  size_t bytesIn=0;
  esp_err_t result = i2s_read(I2S_PORT, &sBuffer, bufferLen, &bytesIn, portMAX_DELAY);
  if (result == ESP_OK)
  {
    int samples_read = bytesIn/bufferLen;
    int16_t sum=0;
    float mean = 0;

    if (samples_read>0)
    {
      for (int i=0; i<samples_read; i++)
      {
        mean += (sBuffer[i]);
      }
      mean/=samples_read; //shorthand for mean=mean/samples_read
      Serial.println(mean);
    }

    
    if (mean>200){
      mouthV = map (mean, 200, 600, 2, 6);
      mouth_open();
      dt = random(0,150);
    }

    else{
      digitalWrite(LED_BUILTIN, HIGH);
      Serial.println ("Voice not detected");
      delay(1000);
    }

    Serial.println(mean);
    delay(100);
  }
}
  
  
}