#include <driver/i2s.h>
#define I2S_WS 8
#define I2S_SCK 9
#define I2S_SD 7
#define I2S_PORT I2S_NUM_0 //choosing I2S0 controller 
#define bufferLen 64

int16_t sBuffer[bufferLen]; //int16_t is data type for 2 byte signed integers

void i2s_install(){

  const i2s_config_t i2s_config = {
    .mode = i2s_mode_t(I2S_MODE_MASTER | I2S_MODE_RX),
    .sample_rate = 44100,
    .bits_per_sample = i2s_bits_per_sample_t(16),
    //Configures I2S to use only the left channel, since we have grounded the L/R pin. To use ..ONLY_RIGHT, leave L/R pin floating
    .channel_format = I2S_CHANNEL_FMT_ONLY_LEFT, 
    .communication_format = i2s_comm_format_t(I2S_COMM_FORMAT_STAND_I2S),
    .intr_alloc_flags = 0, // default interrupt priority
    .dma_buf_count = 8,
    .dma_buf_len = bufferLen,
    .use_apll = false
  };

i2s_driver_install(I2S_PORT, &i2s_config, 0, NULL);
}

void i2s_setpin(){
  const i2s_pin_config_t pin_config = {
    .bck_io_num = I2S_SCK,
    .ws_io_num = I2S_WS,
    .data_out_num = -1,
    .data_in_num = I2S_SD
  };

i2s_set_pin(I2S_PORT, &pin_config);
}

void setup() {
  pinMode(LED_BUILTIN,OUTPUT);

  Serial.begin(115200);
  Serial.println("Setup I2S ...");

  delay(1000);
  i2s_install();
  i2s_setpin();
  i2s_start(I2S_PORT);
  Serial.println("I2S Setup successful ...");
  delay(50);
 
}

void loop() {

  size_t bytesIn=0;
  esp_err_t result = i2s_read(I2S_PORT, &sBuffer, bufferLen, &bytesIn, portMAX_DELAY);
  if (result == ESP_OK)
  {
    int samples_read = bytesIn/bufferLen;
    int16_t sum=0;
    float mean = 0;

    if (samples_read>0)
    {
      for (int i=0; i<samples_read; i++)
      {
        mean += (sBuffer[i]);
      }
      mean/=samples_read; //shorthand for mean=mean/samples_read
      Serial.println(mean);
    }

    if (mean>200){
      digitalWrite(LED_BUILTIN, LOW);
      delay(500);
    }

    else{
      digitalWrite(LED_BUILTIN, HIGH);
    }

    // Serial.print("Amplitude: ");
    Serial.println(mean);
    delay(100);
  }
}
